#import pickle
try:
    from hashlib import sha1
except ImportError:
    from sha import new as sha1
import time
from datetime import datetime, timedelta

from django.db import models
from django.conf import settings
from django.utils import simplejson

from account import settings as app_settings


class AuthKey(models.Model):
    id = models.CharField(max_length=40, primary_key=True)
    uid = models.PositiveIntegerField()
    expired = models.DateTimeField()
    created = models.DateTimeField(auto_now_add=True)
    data = models.TextField()

    def __unicode__(self):
        return 'Key for %s' % self.username

    def export_data(self):
        return simplejson.loads(self.data)

    def import_data(self, **kwargs):
        pairs = kwargs#[]
        #for key, value in kwargs.iteritems():
            #if isinstance(value, unicode):
                #value = value.encode('utf-8')
            #if isinstance(value, datetime):
                #value = strftime(value)
            #pairs.append((key, value))
        self.data = simplejson.dumps(dict(pairs))

    def save(self, *args, **kwargs):
        if not self.id:
            source = '%s%d%d' % (settings.SECRET_KEY, time.time(), id({}))
            self.id = sha1(source).hexdigest()
        if not self.expired:
            self.expired = datetime.now() + timedelta(seconds=app_settings.AUTH_KEY_TIMEOUT)
        super(AuthKey, self).save(*args, **kwargs)


#def strftime(when):
    #return when.strftime('%d-%m-%y-%H-%M-%S')


#def strptime(when):
    #try:
        #return datetime.fromtimestamp(time.mktime(
            #time.strptime(when, '%d-%m-%y-%H-%M-%S')))
    #except ValueError:
        #return None





from django.utils.translation import ugettext_lazy as _

ACCOUNT_USER_MODEL = 'django.contrib.auth.models.User'
ACCOUNT_LOGIN_FORM = 'account.forms.LoginForm'

ACCOUNT_REGISTRATION_REDIRECT_URLNAME = 'registration_complete'
ACCOUNT_REGISTRATION_FORM = 'account.forms.RegistrationForm'
ACCOUNT_REGISTRATION_ENABLED = True
ACCOUNT_ACTIVATION_REQUIRED = True
ACCOUNT_AUTHENTICATION_WITH_EMAIL = False

ACCOUNT_PASSWORD_RESET_FORM = 'account.forms.PasswordResetForm'
ACCOUNT_PASSWORD_CHANGE_FORM = 'account.forms.PasswordChangeForm'
ACCOUNT_PASSWORD_CHANGE_REQUIRES_OLD = True

ACCOUNT_CAPTCHA_ENABLED = False
ACCOUNT_CAPTCHA_FIELD = 'captcha.fields.CaptchaField'
ACCOUNT_CAPTCHA_LABEL = _('Enter the text on the image')



from django.contrib import auth
from django.dispatch import Signal
from django.contrib.sites.models import Site

from urlauth.signals import key_loaded, key_processed

from account.util import email_template


account_created = Signal(providing_args=['user'])


def login_user(request, user):
    user.backend = 'django.contrib.auth.backends.ModelBackend'
    auth.login(request, user)


def key_loaded_handler(key, request):
    extra = key.export_data()
    action = extr.get('action')
    user = request.user

    if 'activation' == action:
        if not user.is_active:
            user.is_active = True
            user.save()
            email_template(user.email, 'account/mail/welcome',
                           user=user, domain=Site.objects.get_current().domain)

    if user.is_active:
        if 'new_email' == action:
            if 'email' in extra:
                user.email = args['email']
                user.save()



from django.conf.urls.defaults import *
import django.contrib.auth.views
from django.views.generic.simple import direct_to_template
from django.contrib.auth.decorators import login_required

from account import views

urlpatterns = patterns('',
    # Common functions
    url(r'^login/$', views.login, name='auth_login'),
    url(r'^logout/$', views.logout, name='auth_logout'),
    url(r'^logout/successful/$', views.logout_successful, name='auth_logout_successful'),

    # Registration
    url(r'^registration/$', views.registration, name='registration_register'),
    url(r'^activation/required/$', direct_to_template, {'template':'account/activation_required.html'}, name='activation_required'),
    url(r'^registration/complete/$', direct_to_template, {'template':'account/registration_complete.html'}, name='registration_complete'),

    # Work with password
    url(r'^password/reset/$', views.password_reset, name='auth_password_reset'),
    url(r'^password/change/$', views.password_change, name='auth_password_change'),
    url(r'^password/change/done/$', views.password_change_done, name='auth_password_change_done'),

    # Work with email
    url(r'^email/change/$', views.email_change, name='auth_email_change'),
    url(r'^email/change/done/$', views.email_change_done, name='auth_email_change_done'),
)



# -*- coding: utf-8

import re

from django import forms
from django.conf import settings
from django.contrib.auth import authenticate, login
from django.utils.translation import ugettext as _
from django.template import loader
from django.contrib.auth.models import User
from django.utils.safestring import mark_safe

from account.util import load_class

UserModel = load_class(settings.ACCOUNT_USER_MODEL)

if settings.ACCOUNT_CAPTCHA_ENABLED:
    CaptchaField = load_class(settings.ACCOUNT_CAPTCHA_FIELD)

RE_USERNAME = getattr(settings, 'ACCOUNT_RE_USERNAME',
                      re.compile(r'[a-z0-9][_a-z0-9]*[a-z0-9]$', re.I))
USERNAME_MIN_LENGTH = getattr(settings, 'ACCOUNT_USERNAME_MIN_LENGTH', 3)
USERNAME_MAX_LENGTH = getattr(settings, 'ACCOUNT_USERNAME_MAX_LENGTH', 20)

PASSWORD_MIN_LENGTH = getattr(settings, 'ACCOUNT_PASSWORD_MIN_LENGTH', 3)
PASSWORD_MAX_LENGTH = getattr(settings, 'ACCOUNT_PASSWORD_MAX_LENGTH', 15)

class PasswordField(forms.CharField):
    """
    Form field for password handling.
    """

    def __init__(self, *args, **kwargs):
        super(PasswordField, self).__init__(*args, **kwargs)
        self.widget = forms.PasswordInput(render_value=False)
        self.help_text = ''


    def clean(self, value):
        super(PasswordField, self).clean(value)
        if len(value) < PASSWORD_MIN_LENGTH:
            raise forms.ValidationError(_(u'Password length is less than %(min)d') % {'min': PASSWORD_MIN_LENGTH})
        if len(value) > PASSWORD_MAX_LENGTH:
            raise forms.ValidationError(_(u'Password length is more than %(max)d') % {'max': PASSWORD_MAX_LENGTH})
        return value


class RegistrationForm(forms.Form):
    username = forms.CharField(label=_(u'Login'), help_text=_(u'You can use a-z, 0-9 and underscore. Login length could be from %(min)s to %(max)s chars.') % {'min': USERNAME_MIN_LENGTH, 'max': USERNAME_MAX_LENGTH})
    email = forms.EmailField(label=_('Email'))
    password = PasswordField(label=_('Password'))
    password_dup = PasswordField(label=_('Password (confirmation)'))

    default_error_messages = {
        'short_login': _(u'Login length is less than %(min)d'),
        'long_login': _(u'Login length is more than %(max)d'),
        'invalid_login': _(u'Login contains restricted symbols'),
        'taken_login': _(u'This login already registered'),
        'taken_email': _(u'This email already registered.'),
        'mismatched_passwords': _('Passwords do not match'),
    }

    def __init__(self, *args, **kwargs):
        super(RegistrationForm, self).__init__(*args, **kwargs)
        if settings.ACCOUNT_CAPTCHA_ENABLED:
            self.fields['captcha'] = CaptchaField(label=settings.ACCOUNT_CAPTCHA_LABEL)


    def clean_username(self):
        if 'username' in self.cleaned_data:
            value = self.cleaned_data['username']
            if len(value) < USERNAME_MIN_LENGTH:
                raise forms.ValidationError(self.default_error_messages['short_login'] % {'min': USERNAME_MIN_LENGTH})
            if len(value) > USERNAME_MAX_LENGTH:
                raise forms.ValidationError(self.default_error_messages['long_login'] % {'max': USERNAME_MAX_LENGTH})
            if not RE_USERNAME.match(value):
                raise forms.ValidationError(self.default_error_messages['invalid_login'])

            try:
                UserModel.objects.get(username__exact=value)
            except UserModel.DoesNotExist:
                return value
            else:
                raise forms.ValidationError(self.default_error_messages['taken_login'])

    
    def clean_email(self):
        #return self.cleaned_data.get('email','')
        if 'email' in self.cleaned_data:
            email = self.cleaned_data['email']
            try:
                UserModel.objects.get(email__exact=email)
            except UserModel.DoesNotExist:
                return email
            else:
                raise forms.ValidationError(self.default_error_messages['taken_email'])

    
    def clean(self):
        pwd1 = self.cleaned_data.get('password')
        pwd2 = self.cleaned_data.get('password_dup')
        if pwd1 and pwd2:
            if pwd1 != pwd2:
                # show error on top of password_dup field
                self._errors['password_dup'] = [self.default_error_messages['mismatched_passwords']]
        return self.cleaned_data


    def save(self):
        username = self.cleaned_data['username']
        email = self.cleaned_data['email']
        password = self.cleaned_data['password']
        user = UserModel.objects.create_user(username, email, password)
        if settings.ACCOUNT_ACTIVATION_REQUIRED:
            user.is_active = False
            user.save()
        return user


class PasswordResetForm(forms.Form):
    email = forms.EmailField(label=_('Email'))

    default_error_messages = {'taken_email': _(u'This email is not registered')}

    def clean_email(self):
        if 'email' in self.cleaned_data:
            email = self.cleaned_data['email']
            if UserModel.objects.filter(email=email).count():
                return email
            else:
                raise forms.ValidationError(self.default_error_messages['taken_email'])


class LoginForm(forms.Form):
    username = forms.CharField(label=_('Username'))
    password = forms.CharField(label=_('Password'), widget=forms.PasswordInput)

    default_error_messages = {
        'invalid_account': _(u'Incorrect login or password'),
        'inactive_account': _(u'Sorry. You account is not active. Maybe you didn\'t activate it.'),
    }

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request')
        self.base_fields['username'].help_text = ''
        #self.base_fields['password'].widget = forms.PasswordInput()
        self.base_fields['password'].help_text = ''
        super(LoginForm, self).__init__(*args, **kwargs)


    def clean(self):
        super(LoginForm, self).clean()
        if self.is_valid():
            user = authenticate(
                username=self.cleaned_data['username'],
                password=self.cleaned_data['password'])
            if not user is None:
                if user.is_active:
                    login(self.request, user)
                    return self.cleaned_data
                else:
                    raise forms.ValidationError(self.default_error_messages['inactive_account'])
            else:
                raise forms.ValidationError(self.default_error_messages['invalid_account'])


# TODO: REWRITE THIS HELL HORROR

class PasswordChangeForm(forms.Form):
    """
    Form for changing user's password.
    """

    old_password = PasswordField(label=_(u'Old password'))
    password = PasswordField(label=_(u'Password'))
    password_confirmation = PasswordField(label=_(u'Password (confirmation)'))
    authkey = forms.CharField(widget=forms.HiddenInput, required=False)
    uid = forms.CharField(widget=forms.HiddenInput, required=False)

    default_error_messages = {
        'invalid_password': _('Incorrect old password'),
        'mismatched_passwords': _(u'The passwords do not match'),
    }

    def __init__(self, *args, **kwargs):
        self.require_old = kwargs.pop('require_old', True)
        self.user = kwargs.pop('user')
        if not self.require_old:
            self.base_fields['old_password'] = forms.Field(widget=forms.HiddenInput, required=False)
        super(PasswordChangeForm, self).__init__(*args, **kwargs)


    def clean(self):
        if 'old_password' in self.cleaned_data and 'uid' in self.cleaned_data:
            password = self.cleaned_data['old_password']
            uid = self.cleaned_data['uid']
            if password:
                test_user = authenticate(username=self.user.username, password=password)
                if not test_user:
                    del self.cleaned_data['old_password']
                    self._errors['old_password'] = [self.default_error_messages['invalid_password']]
        return self.cleaned_data


    def clean_password_confirmation(self):
        pass1 = self.cleaned_data['password']
        pass2 = self.cleaned_data['password_confirmation']
        if pass1 != pass2:
            raise forms.ValidationError(self.default_error_messages['mismatched_passwords'])
        else:
            return pass1


    def clean_uid(self):
        if self.require_old:
            return self.cleaned_data['uid']
        else:
            try:
                user = User.objects.get(pk=self.cleaned_data['uid'])
                self.user = user
                return self.cleaned_data['uid']
            except User.DoesNotExist:
                raise forms.ValidationError(_(u'Invalid username'))


    def save(self):
        self.user.set_password(self.cleaned_data['password'])
        self.user.save()
        return self.user


class EmailChangeForm(forms.Form):
    """
    Form for email chanage.
    """

    email = forms.EmailField(label=_(u'New email'))



from datetime import datetime, timedelta
import pickle
import cgi

from django.test import TestCase
from django.test.client import Client
from django.conf.urls.defaults import *
from django.http import HttpResponse
from django.conf import settings
from django.contrib.auth import login, authenticate

from urlauth.util import wrap_url
from account.util import str_to_class
from account import settings as app_settings

UserModel = str_to_class(app_settings.USER_MODEL)

class AccountTestCase(TestCase):
    urls = 'account.tests'

    def setUp(self):
        UserModel.objects.all().delete()
        u = UserModel(username='user', email='user@host.com')
        u.set_password('pass')
        u.is_active = True
        u.save()
        self.user = u

        u = UserModel(username='ban_user', email='ban_user@host.com')
        u.set_password('pass')
        u.is_active = False
        u.save()
        self.ban_user = u


class SignalsTestCase(AccountTestCase):

    def testActivation(self): 
        def process_url(url, **kwargs):
           url = wrap_url(url, **kwargs)
           return url.split('?')[0], cgi.parse_qs(url.split('?')[1])

        test_url = '/account_test_view/'
        expired = datetime.now() + timedelta(days=1)
        resp = self.client.get(test_url)


        # Activation of baned user
        self.client.logout()
        url, args = process_url(test_url, uid=self.ban_user.id, expired=expired, action='activation')
        resp = self.client.get(url, args)
        self.assertEqual(self.client.session['_auth_user_id'], self.ban_user.id)
        self.client.session.flush()

        # New password
        #self.client.logout()
        #url, args = process_url(test_url, uid=self.user.id, expired=expired,
                                #action='new_password', password='foobar')
        #self.assertTrue(authenticate(uid=self.user.id, password='pass'))
        #resp = self.client.get(url, args)
        #self.assertEqual(self.client.session['_auth_user_id'], self.user.id)
        #self.client.session.flush()
        #self.assertTrue(authenticate(uid=self.user.id, password='foobar'))
        #self.client.session.flush()

        # New email
        self.client.logout()
        expired = datetime.now() + timedelta(days=1)
        url, args = process_url(test_url, uid=self.user.id, expired=expired,
                                action='new_email', email='user_gaz@host.com')
        self.assertEqual('user@host.com', self.user.email)
        resp = self.client.get(url, args)
        self.assertEqual(self.client.session['_auth_user_id'], self.user.id)
        user = UserModel.objects.get(pk=self.user.id)
        self.assertEqual(user.email, 'user_gaz@host.com')


def test_view(request):
    return HttpResponse()

urlpatterns = patterns('',
    url('account_test_view/', test_view, name='account_test_view'),
)



# -*- coding: utf-8
from datetime import datetime, timedelta

from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.contrib import auth
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.contrib.sites.models import Site
from django.shortcuts import redirect

from urlauth.util import wrap_url

from account.forms import PasswordResetForm,\
                          PasswordChangeForm, LoginForm, EmailChangeForm
from account.util import email_template, build_redirect_url, render_to, load_class
from account import signals


RegistrationForm = load_class(settings.ACCOUNT_REGISTRATION_FORM)
LoginForm = load_class(settings.ACCOUNT_LOGIN_FORM)
PasswordResetForm = load_class(settings.ACCOUNT_PASSWORD_RESET_FORM)
ChangePasswordForm = load_class(settings.ACCOUNT_PASSWORD_CHANGE_FORM)
UserModel = load_class(settings.ACCOUNT_USER_MODEL)
hostname = Site.objects.get_current().domain


@render_to('account/message.html')
def message(request, msg):
    """
    Shortcut that prepare data for message view.
    """

    return {'message': msg,
            }


@render_to('account/registration.html')
def registration(request, form_class=RegistrationForm):
    if not settings.ACCOUNT_REGISTRATION_ENABLED:
        return message(request, _('Sorry. Registration is disabled.'))
    if request.user.is_authenticated():
        return message(request, _('You have to logout before registration'))

    if 'POST' == request.method:
        form = form_class(request.POST, request.FILES)
    else:
        form = form_class()

    if form.is_valid():
        user = form.save()

        signals.account_created.send(None, user=user)
        password = form.cleaned_data['password']

        if settings.ACCOUNT_ACTIVATION_REQUIRED:
            url = 'http://%s%s' % (hostname, reverse('registration_complete'))
            url = wrap_url(url, uid=user.id, action='activation')
            params = {'domain': hostname, 'login': user.username, 'url': url,
                      'password': password}
            if email_template(user.email, 'account/mail/activation_required', **params):
                return HttpResponseRedirect(reverse('activation_required'))
            else:
                user.delete()
                return message(request, _('The error was occuried while sending email with activation code. Account was not created. Please, try later.'))
        else:
            user.backend = 'django.contrib.auth.backends.ModelBackend'
            auth.login(request, user)    
            args = {'domain': hostname, 'user': user, 'password': password}
            email_template(user.email, 'account/mail/registration_complete', **args)
            return redirect(reverse(settings.ACCOUNT_REGISTRATION_REDIRECT_URLNAME))

    return {'form': form,
            }


@render_to('account/password_reset.html')
def password_reset(request, form_class=PasswordResetForm):
    if 'POST' == request.method:
        form = form_class(request.POST)
    else:
        form = form_class()

    if form.is_valid():
        user = UserModel.objects.get(email=form.cleaned_data['email'])
        url = 'http://%s%s' % (hostname, reverse('auth_password_change'))

        url = wrap_url(url, uid=user.id, onetime=False, action='password_change')
        args = {'domain': hostname, 'url': url, 'user': user}
        if email_template(user.email, 'account/mail/password_reset', **args):
            return message(request, _('Check the mail please'))
        else:
            return message(request, _('Unfortunately we could not send you email in current time. Please, try later'))

    return {'form': form,
            }


@render_to('account/login.html')
def login(request, form_class=LoginForm):
    if request.user.is_authenticated():
        return message(request, _('You are already authenticated'))

    if 'POST' == request.method:
        form = form_class(request.POST, request=request)
    else:
        form = form_class(request=request)

    request.session['login_redirect_url'] = request.GET.get('next')
    if form.is_valid():
        redirect_url = build_redirect_url(request, settings.LOGIN_REDIRECT_URL)
        return HttpResponseRedirect(redirect_url)
    return {'form': form,
            }


@render_to('account/password_change.html')
def password_change(request):
    """
    That view is used in two cases:
     * User is authenticated. He fills the from with old password new one.
     * User follow the link from reset password email. In that case field for old password is invisible.
    """

    authkey = None
    if hasattr(request, 'authkey'):
        if request.authkey.extra.get('action') == 'password_change':
            authkey = request.authkey


    if not request.user.is_authenticated():
        if not authkey:
            return HttpResponseRedirect(reverse('auth_login') + '?next=%s' % request.path)
    
    if authkey:
        require_old = False
        initial = {'authkey': authkey.id, 'uid': authkey.uid}
    else:
        require_old = True
        initial = {}

    if 'POST' == request.method:
        form = ChangePasswordForm(request.POST, require_old=require_old, user=request.user)
    else:
        form = ChangePasswordForm(require_old=require_old, initial=initial, user=request.user)

    if form.is_valid():
        form.save()
        if authkey:
            authkey.delete()
        return HttpResponseRedirect(reverse('auth_password_change_done')) 

    return {'form': form,
            }


@login_required
@render_to('account/email_change.html')
def email_change(request):
    if 'POST' == request.method:
        form = EmailChangeForm(request.POST)
    else:
        form = EmailChangeForm()

    if form.is_valid():
        email = form.cleaned_data['email']
        url = 'http://%s%s' % (hostname, reverse('auth_email_change_done'))
        url = wrap_url(url, uid=request.user.id, action='new_email', email=email)
        args = {'domain': hostname, 'url': url, 'email': email,}
        if email_template(email, 'account/mail/email_change', **args):
            return message(request, _('Check the mail please'))
        else:
            return message(request, _('Unfortunately we could not send you email in current time. Please, try later'))
    return {'form': form,
            }


@login_required
def email_change_done(request):
    return message(request, _('Your email has been changed to %s') % request.user.email)


@render_to('account/password_change_done.html')
def password_change_done(request):
    return {'login_url': reverse('auth_login'),
            }


def logout(request):
    auth.logout(request)
    next = request.GET.get('next', reverse('auth_logout_successful'))
    return HttpResponseRedirect(next)


def logout_successful(request):
    return message(request, _('You have successfully loged out'))



# -*- coding: utf-8
"""
Useful utilities for account application.
"""

import re
import os.path
from datetime import datetime
import logging

from django.conf import settings
from django.template.loader import get_template
from django.template import Context, RequestContext
from django.shortcuts import render_to_response
from django.core.mail import send_mail
from django.conf import settings
from django.contrib.sites.models import Site

from django.core.mail import EmailMultiAlternatives


def build_redirect_url(request, default_url):
    """
    Retrieve redirect url from session.
    
    Use default if retrieved one is broken or not safe.
    """

    url = request.session.get('login_redirect_url')
    if not url or '//' in url or ' ' in url:
        url = default_url
    try:
        del request.session['login_redirect_url']
    except KeyError:
        pass
    return url


def parse_template(template_path, **kwargs):
    """
    Load and render template.

    First line of template should contain the subject of email.
    Return tuple with subject and content.
    """

    template = get_template(template_path)
    context = Context(kwargs)
    re_empty_lines = re.compile(r'^(\r?\n)+|(\r?\n)+$')
    data = template.render(context)
    return re_empty_lines.sub('', data)


def email_template(rcpt, template_path, **kwargs):
    """
    Load, render and email template.

    Template_path should not contain .txt or .html suffixes - they
    will be appended automatically.

    **kwargs may contain variables for template rendering.
    """

    from_email = settings.DEFAULT_FROM_EMAIL

    subject = parse_template('%s_subject.txt' % template_path, **kwargs)
    text_content = parse_template('%s_body.txt' % template_path, mode='text', **kwargs)
    html_content = parse_template('%s_body.txt' % template_path, mode='html', **kwargs)

    msg = EmailMultiAlternatives(subject, text_content, from_email, [rcpt])
    msg.attach_alternative(html_content, "text/html")

    return bool(msg.send(fail_silently=True))


def render_to(template_path):
    """
    Decorate the django view.

    Wrap view that return dict of variables, that should be used for
    rendering the template.
    Dict returned from view could contain special keys:
     * MIME_TYPE: mimetype of response
     * TEMPLATE: template that should be used insted one that was
                 specified in decorator argument
    """

    def decorator(func):
        def wrapper(request, *args, **kwargs):
            output = func(request, *args, **kwargs)
            if not isinstance(output, dict):
                return output
            kwargs = {'context_instance': RequestContext(request)}
            if 'MIME_TYPE' in output:
                kwargs['mimetype'] = output.pop('MIME_TYPE')

            template = template_path
            if 'TEMPLATE' in output:
                template = output.pop('TEMPLATE')
            return render_to_response(template, output, **kwargs)
        return wrapper
    return decorator


def load_class(path):
    from django.db.models.loading import get_app
    module_path, class_name = path.rsplit('.', 1)
    mod = __import__(module_path, globals(), locals(), ['foobar'])
    return getattr(mod, class_name)


def build_absolute_url(url):
    return 'http://%s%s' % (Site.objects.get_current().domain, url)
